@@ -1,9 +1,11 @@
 # -*- coding: utf-8 -*-
+from datetime import date, datetime, time
+import json
+import sys
 try:
     import cPickle as pickle
 except:
     import pickle
-import json
 import urllib
 import weakref
 import warnings
@@ -29,6 +31,8 @@ from traversals import TraversalDescription
 __all__ = ["GraphDatabase", "Incoming", "Outgoing", "Undirected",
            "StopAtDepth", "NotFoundError", "StatusException", "Q"]
 
+PYTHON_VERSION = sys.version_info
+
 
 class StopAtDepth(object):
     """
@@ -700,6 +704,25 @@ class Base(object):
 
     @staticmethod
     def _safe_string(s):
+        if options.SMART_DATES:
+            if isinstance(s, (datetime, date, time)):
+                return s
+            else:
+                try:
+                    dt = datetime.strptime(s, options.DATETIME_FORMAT)
+                except ValueError:
+                    for date_type in ["date", "time"]:
+                        try:
+                            option = "%s_FORMAT" % date_type.upper()
+                            format = getattr(options, option)
+                            t = getattr(datetime.strptime(s, format),
+                                        date_type)()
+                        except ValueError:
+                            pass
+                        else:
+                            return t
+                else:
+                    return dt
         if isinstance(s, unicode):
             return s
         if isinstance(s, basestring):
@@ -762,7 +785,10 @@ class Base(object):
             else:
                 raise NotFoundError(response.status,
                                     "Node or propery not found")
-        return self._dic["data"][key]
+        if options.SMART_DATES:
+            return Base._safe_string(self._dic["data"][key])
+        else:
+            return self._dic["data"][key]
 
     def get(self, key, *args, **kwargs):
         tx = kwargs.get("tx", None)
@@ -801,7 +827,10 @@ class Base(object):
             response, content = Request(**self._auth).put(property_url,
                                                           data=value)
             if response.status == 204:
-                self._dic["data"].update({key: value})
+                if options.SMART_DATES:
+                    self._dic["data"].update({key: Base._safe_string(value)})
+                else:
+                    self._dic["data"].update({key: value})
             elif response.status == 404:
                 raise NotFoundError(response.status, "Node or property not found")
             else:
@@ -875,6 +904,8 @@ class Base(object):
             return u"<Neo4j %s: %s>" % (self.__class__.__name__, self.url)
 
     def _get_properties(self):
+        if options.SMART_DATES:
+            self._update_dict_data()
         return self._dic["data"]
 
     def _set_properties(self, props={}):
@@ -1526,6 +1557,88 @@ class Index(object):
                             name=self.name, auth=self._auth,
                             cypher=self._cypher, key=key, tx=tx)
 
+    def get_or_create(self, key, value, item=None, properties=None,
+                      node_from=None, relationship_name=None, node_to=None,
+                      tx=None):
+        return self._uniqueness("get_or_create", key=key, value=value,
+                                item=item, properties=properties,
+                                node_from=node_from,
+                                relationship_name=relationship_name,
+                                node_to=node_to, tx=tx)
+
+    def create_or_fail(self, key, value, item=None, properties=None,
+                       node_from=None, relationship_name=None, node_to=None,
+                       tx=None):
+        return self._uniqueness("create_or_fail", key=key, value=value,
+                                item=item, properties=properties,
+                                node_from=node_from,
+                                relationship_name=relationship_name,
+                                node_to=node_to, tx=tx)
+
+    def _uniqueness(self, uniqueness, key, value, item=None, properties=None,
+                    node_from=None, relationship_name=None, node_to=None,
+                    tx=None):
+        url = "%s?uniqueness=%s" % (self.url, uniqueness)
+        if item:
+            properties = item.properties()
+        elif not properties:
+            properties = {}
+        if self._index_for == NODE:
+            data = {
+                "key": key,
+                "value": value,
+                "properties": properties,
+            }
+        else:
+            data = {
+                "key": key,
+                "value": value,
+            }
+            if properties:
+                data.update({"properties": properties})
+            if item:
+                data.update({
+                    "start": item.start,
+                    "end": item.end,
+                    "type": item.type,
+                })
+            else:
+                data.update({
+                    "start": node_from,
+                    "end": node_to,
+                    "type": relationship_name,
+                })
+        tx = Transaction.get_transaction(tx)
+        if tx:
+            op = tx.subscribe(TX_POST, url, data=data,
+                              obj=self, returns=self._index_for)
+            return op
+        else:
+            index_for = self._index_for.capitalize()
+            request = Request(**self._auth)
+            response, content = request.post(url, data=data)
+            if response.status in [200, 201]:
+                # Returns object that was indexed
+                entity = json.loads(content)
+                if self._index_for == NODE:
+                    return Node(entity['self'], data=entity['data'],
+                                auth=self._auth, update_dict=entity)
+                else:
+                    return Relationship(entity['self'],
+                                        data=entity['data'],
+                                        auth=self._auth)
+            elif response.status == 409:  # Create or fail, failing
+                if options.SMART_ERRORS:
+                    raise ValueError("duplicated item in index '%s'"
+                                     % index_for)
+                else:
+                    raise StatusException(response.status, "Duplicated item")
+            else:
+                if options.SMART_ERRORS:
+                    raise KeyError(index_for)
+                else:
+                    raise StatusException(response.status, "Duplicated item")
+
     def delete(self, key=None, value=None, item=None, tx=None):
         if not key and not value and not item:
             url = self.template.replace("/{key}/{value}", "")
@@ -1552,7 +1665,7 @@ class Index(object):
                 request_url = "index/%s/%s/%s" % (self._index_for, self.name,
                                                   item.id)
             else:
-                raise TypeError("delete() takes at least 2 arguments, the " \
+                raise TypeError("delete() takes at least 2 arguments, the "
                                 "key of the index and the %s to remove"
                                 % self._index_for)
         tx = Transaction.get_transaction(tx)
@@ -2169,12 +2282,25 @@ def elements_filter(cls, lookups=[], start=None, returns=None):
         raise CypherException
 
 
-def smart_quote(val):
-    if isinstance(val, (bool, int, float, long)):
-        return urllib.quote(json.dumps(val), safe="")
-    else:
-        try:
-            safe_key = urllib.quote(val, safe="")
-        except (KeyError, UnicodeEncodeError, UnicodeError):
-            safe_key = urllib.quote(val.encode("utf8"), safe="")
-        return safe_key
+if PYTHON_VERSION < (2, 7):
+    def smart_quote(val):
+        if isinstance(val, (bool, int, long)):
+            return urllib.quote(json.dumps(val), safe="")
+        elif isinstance(val, float):
+            return unicode(val)
+        else:
+            try:
+                safe_key = urllib.quote(val, safe="")
+            except (KeyError, UnicodeEncodeError, UnicodeError):
+                safe_key = urllib.quote(val.encode("utf8"), safe="")
+            return safe_key
+else:
+    def smart_quote(val):
+        if isinstance(val, (bool, int, float, long)):
+            return urllib.quote(json.dumps(val), safe="")
+        else:
+            try:
+                safe_key = urllib.quote(val, safe="")
+            except (KeyError, UnicodeEncodeError, UnicodeError):
+                safe_key = urllib.quote(val.encode("utf8"), safe="")
+            return safe_key
