@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from datetime import datetime
+from functools import wraps
 try:
     import cPickle as pickle
 except:
@@ -16,6 +17,27 @@ import request
 
 
 NEO4J_URL = os.environ.get('NEO4J_URL', "http://localhost:7474/db/data/")
+NEO4J_VERSION = os.environ.get('NEO4J_VERSION', None)
+
+
+def versions(supported=None, not_supported=None):
+    supported = supported or []
+    not_supported = not_supported or []
+
+    def _versions(fn):
+
+        @wraps(fn)
+        def wrapper(*args, **kwargs):
+            if NEO4J_VERSION:
+                if ((supported and NEO4J_VERSION in supported)
+                        or (not_supported
+                            and NEO4J_VERSION not in not_supported)):
+                    fn(*args, **kwargs)
+            else:
+                fn(*args, **kwargs)
+        return wrapper
+    return _versions
+
 
 class NodesTestCase(unittest.TestCase):
     def setUp(self):
@@ -82,6 +104,25 @@ class NodesTestCase(unittest.TestCase):
         n = self.gdb.node(name="John Doe", profession="Hacker")
         self.assertEqual(n.get("surname", "Doe"), "Doe")
 
+    def test_create_node_date(self):
+        import options as clientSmartDates
+        clientSmartDates.SMART_DATES = True
+        dt = datetime.utcnow()
+        d = dt.date()
+        t = dt.time()
+        n = self.gdb.nodes.create(name="John Doe", datetime=dt, date=d, time=t)
+        self.assertEqual(n.properties, {"name": "John Doe",
+                                        "datetime": dt,
+                                        "date": d,
+                                        "time": t})
+        self.assertEqual(n["datetime"], dt)
+        self.assertEqual(n["date"], d)
+        self.assertEqual(n["time"], t)
+        self.assertEqual(n.get("datetime"), dt)
+        self.assertEqual(n.get("date"), d)
+        self.assertEqual(n.get("time"), t)
+        clientSmartDates.SMART_DATES = False
+
     def test_get_node(self):
         n1 = self.gdb.nodes.create(name="John Doe", profession="Hacker")
         n2 = self.gdb.nodes.get(n1.id)
@@ -119,6 +160,17 @@ class NodesTestCase(unittest.TestCase):
         n2 = self.gdb.nodes.get(n1.id)
         self.assertEqual(n1.get("name"), n2.get("name"))
 
+    def test_set_node_property_date(self):
+        import options as clientSmartDates
+        clientSmartDates.SMART_DATES = True
+        dt = datetime.utcnow()
+        n1 = self.gdb.node(name="John Doe", profession="Hacker")
+        n1.set("birthdate", dt)
+        n2 = self.gdb.nodes.get(n1.id)
+        self.assertEqual(n1.get("birthdate"), dt)
+        self.assertEqual(n2.get("birthdate"), dt)
+        clientSmartDates.SMART_DATES = False
+
     def test_set_node_properties(self):
         n1 = self.gdb.node(name="John Doe", profession="Hacker")
         n1.properties = {"name": "Jimmy Doe"}
@@ -134,7 +186,8 @@ class NodesTestCase(unittest.TestCase):
     def test_set_node_property_safe(self):
         n1 = self.gdb.node(language="Español antigüillo")
         n1.set("Idioma de los subtítulos", "Español antigüillo")
-        self.assertEqual(n1.get("Idioma de los subtítulos"), n1.get("language"))
+        self.assertEqual(n1.get("Idioma de los subtítulos"),
+                         n1.get("language"))
 
     def test_set_node_properties_safe(self):
         n1 = self.gdb.node()
@@ -391,6 +444,60 @@ class IndexesTestCase(RelationshipsTestCase):
         results = index.query(-Q('surnames', 'donald') | +Q('place', 'Texas'))
         self.assertTrue(n2 not in results and n1 in results)
 
+    @versions(not_supported=["1.6.3", "1.7.2", "1.8.2"])
+    def test_index_get_or_create_created(self):
+        index = self.gdb.nodes.indexes.create(name="doe")
+        properties = {
+            "name": "Lemmy",
+            "band": "Motörhead",
+        }
+        n1 = index.get_or_create(key="bands", value="Motörhead",
+                                 properties=properties)
+        self.assertTrue(n1 in index["bands"]["Motörhead"])
+
+    @versions(not_supported=["1.6.3", "1.7.2", "1.8.2"])
+    def test_index_get_or_create_existing(self):
+        index = self.gdb.nodes.indexes.create(name="doe")
+        now = datetime.now().strftime('%s%f')
+        properties = {
+            "name": "Lemmy",
+            "band": "Motörhead",
+            "now": now,
+        }
+        n1 = self.gdb.nodes.create(**properties)
+        index["now"][now] = n1
+        n2 = index.get_or_create(key="now", value=now,
+                                 properties=properties)
+        self.assertEqual(n1, n2)
+        self.assertTrue(n1 in index["now"][now])
+        self.assertTrue(n2 in index["now"][now])
+
+    @versions(not_supported=["1.6.3", "1.7.2", "1.8.2"])
+    def test_index_create_or_fail_created(self):
+        index = self.gdb.nodes.indexes.create(name="doe")
+        properties = {
+            "name": "Lemmy",
+            "band": "Motörhead",
+        }
+        n1 = index.create_or_fail(key="bands", value="Motörhead",
+                                  properties=properties)
+        self.assertTrue(n1 in index["bands"]["Motörhead"])
+
+    @versions(not_supported=["1.6.3", "1.7.2", "1.8.2"])
+    def test_index_create_or_fail_existing(self):
+        index = self.gdb.nodes.indexes.create(name="doe")
+        now = datetime.now().strftime('%s%f')
+        properties = {
+            "name": "Lemmy",
+            "band": "Motörhead",
+            "now": now,
+        }
+        n1 = self.gdb.nodes.create(**properties)
+        index["now"][now] = n1
+        self.assertRaises((Exception, ValueError, request.StatusException),
+                          index.create_or_fail,
+                          key="now", value=now, properties=properties)
+
 
 class TraversalsTestCase(IndexesTestCase):
 
@@ -436,7 +543,8 @@ class TraversalsTestCase(IndexesTestCase):
         # Test from @shahin: https://gist.github.com/1418704
         n1 = self.gdb.node()
         n2 = self.gdb.node()
-        rel = self.gdb.relationships.create(n1,'knows',n2)
+        rel = self.gdb.relationships.create(n1, 'knows', n2)
+
         # Define path traverser
         class PathTraverser(self.gdb.Traversal):
             stop = constants.STOP_AT_END_OF_GRAPH
@@ -483,11 +591,11 @@ class TraversalsTestCase(IndexesTestCase):
         n1.KNOWS(n2)
         n2.KNOWS(n3)
         #all traversal
-        traversal = n1.traverse(stop=3,\
+        traversal = n1.traverse(stop=3,
                                 returnable=constants.RETURN_ALL_NODES)
         self.assertEqual(len(traversal), 3)
         #all but start traversal
-        traversal = n1.traverse(stop=3,\
+        traversal = n1.traverse(stop=3,
                                 returnable=constants.RETURN_ALL_BUT_START_NODE)
         self.assertEqual(len(traversal), 2)
         #custom javascript return filter
@@ -558,6 +666,8 @@ class TraversalsTestCase(IndexesTestCase):
             target = self.gdb.node(message='world')
             relationship = self.source.related_to(target, message="graphy")
             secondrel = target.likes(self.source, message="buh")
+            relationship
+            secondrel
 
     def test_traverse_string_types(self):
         self.create_data()
@@ -577,7 +687,7 @@ class TraversalsTestCase(IndexesTestCase):
         from traversals import RelationshipDirection
         OUTGOING = RelationshipDirection.OUTGOING
         INCOMING = RelationshipDirection.INCOMING
-        ANY = RelationshipDirection.ANY
+        #ANY = RelationshipDirection.ANY
         traverser = db.traversal()\
             .relationships('related_to', OUTGOING)\
             .traverse(start_node)
@@ -657,7 +767,7 @@ class TraversalsTestCase(IndexesTestCase):
             self.assertNotEqual(start_node, None)
             self.assertNotEqual(end_node, None)
             # START SNIPPET: accessPathLastRelationship
-            last_relationship = path.last_relationship
+            #last_relationship = path.last_relationship
             # END SNIPPET: accessPathLastRelationship
             # START SNIPPET: loopThroughPath
             for item in path:
@@ -693,6 +803,7 @@ class ExtensionsTestCase(TraversalsTestCase):
             fail = True
         self.assertTrue(not fail)
 
+    @versions(supported=["1.6.3", "1.7.2", "1.8.2", "1.9.0"])
     def test_gremlin_extension_reference_node(self):
         # Assuming the GremlinPlugin installed
         ext = self.gdb.extensions.GremlinPlugin
@@ -700,6 +811,7 @@ class ExtensionsTestCase(TraversalsTestCase):
         gremlin_n = ext.execute_script(script='g.v(%s)' % n.id)
         self.assertEqual(gremlin_n, n)
 
+    @versions(supported=["1.6.3", "1.7.2", "1.8.2", "1.9.0"])
     def test_gremlin_extension_reference_node_returns(self):
         # Assuming the GremlinPlugin installed
         ext = self.gdb.extensions.GremlinPlugin
@@ -708,6 +820,7 @@ class ExtensionsTestCase(TraversalsTestCase):
                                        returns=constants.NODE)
         self.assertEqual(gremlin_n, n)
 
+    @versions(supported=["1.6.3", "1.7.2", "1.8.2", "1.9.0"])
     def test_gremlin_extension_relationships(self):
         # Assuming the GremlinPlugin installed
         n1 = self.gdb.nodes.create()
@@ -721,6 +834,7 @@ class ExtensionsTestCase(TraversalsTestCase):
         for rel in rels:
             self.assertTrue(isinstance(rel, client.Relationship))
 
+    @versions(supported=["1.6.3", "1.7.2", "1.8.2", "1.9.0"])
     def test_gremlin_extension_relationships_returns(self):
         # Assuming the GremlinPlugin installed
         n1 = self.gdb.nodes.create()
@@ -738,6 +852,7 @@ class ExtensionsTestCase(TraversalsTestCase):
             self.assertTrue(isinstance(rel, client.Relationship))
         clientDebug.DEBUG = False
 
+    @versions(supported=["1.6.3", "1.7.2", "1.8.2", "1.9.0"])
     def test_gremlin_extension_reference_raw_returns(self):
         # Assuming the GremlinPlugin installed
         ext = self.gdb.extensions.GremlinPlugin
@@ -747,6 +862,7 @@ class ExtensionsTestCase(TraversalsTestCase):
         self.assertEqual(gremlin_n["data"], n.properties)
         self.assertTrue(isinstance(gremlin_n, dict))
 
+    @versions(supported=["1.6.3", "1.7.2", "1.8.2", "1.9.0"])
     def test_gremlin_results_raw(self):
         # Assuming the GremlinPlugin installed
         ext = self.gdb.extensions.GremlinPlugin
@@ -754,6 +870,7 @@ class ExtensionsTestCase(TraversalsTestCase):
         self.assertTrue(isinstance(n, list))
         self.assertEqual(n, [1, 2])
 
+
 class TransactionsTestCase(ExtensionsTestCase):
 
     def test_transaction_delete(self):
@@ -903,17 +1020,17 @@ class TransactionsTestCase(ExtensionsTestCase):
         n["age"] = 25
         n["name"] = "John"
         n["place"] = "Houston"
-        with self.gdb.transaction(commit=False, using_globals=False) as tx1:
-            with self.gdb.transaction(commit=False, using_globals=False) as tx2:
-                n.delete("age", tx=tx1)
-                n["name"] = tx2("Jonathan")
-                n["place", tx2] = "Toronto"
+        with self.gdb.transaction(commit=False, using_globals=False) as t1:
+            with self.gdb.transaction(commit=False, using_globals=False) as t2:
+                n.delete("age", tx=t1)
+                n["name"] = t2("Jonathan")
+                n["place", t2] = "Toronto"
         self.assertTrue("age" in n.properties)
-        tx1.commit()
+        t1.commit()
         self.assertTrue("age" not in n.properties)
         self.assertEqual(n["name"], "John")
         self.assertEqual(n["place"], "Houston")
-        tx2.commit()
+        t2.commit()
         self.assertEqual(n["name"], "Jonathan")
         self.assertEqual(n["place"], "Toronto")
 
@@ -930,7 +1047,7 @@ class TransactionsTestCase(ExtensionsTestCase):
         tx.commit()
         self.assertEqual(initial_rels + rels_number, len(n1.relationships))
         self.assertTrue(all([isinstance(r, client.Relationship)]
-                             for r in relations))
+                            for r in relations))
 
     def test_transaction_dict(self):
         nodes = {}
@@ -1025,14 +1142,15 @@ class TransactionsTestCase(ExtensionsTestCase):
         n1 = self.gdb.nodes.create()
         index = self.gdb.nodes.indexes.create('index_nodes')
         with self.gdb.transaction():
-            index.add('test1','test1', n1)
+            index.add('test1', 'test1', n1)
             index['test2']['test2'] = n1
         self.assertTrue(index['test1']['test1'][-1] == n1)
         self.assertTrue(index['test2']['test2'][-1] == n1)
 
     def test_transaction_index_add_rel_to_index(self):
         """
-        Tests whether a relationship can be added to an index within a transaction.
+        Tests whether a relationship can be added to an index within a
+        transaction.
         Does not assert transactionality.
         """
         #test nodes
@@ -1041,7 +1159,7 @@ class TransactionsTestCase(ExtensionsTestCase):
         r = n1.relationships.create('Knows', n2)
         index = self.gdb.relationships.indexes.create('index_rel')
         with self.gdb.transaction():
-            index.add('test1','test1', r)
+            index.add('test1', 'test1', r)
             index['test2']['test2'] = r
         self.assertTrue(index['test1']['test1'][-1] == r)
         self.assertTrue(index['test2']['test2'][-1] == r)
@@ -1053,7 +1171,7 @@ class TransactionsTestCase(ExtensionsTestCase):
         """
         n1 = self.gdb.nodes.create()
         index = self.gdb.nodes.indexes.create('index2')
-        index.add('test2','test2', n1)
+        index.add('test2', 'test2', n1)
         # Test getting nodes from index during transaction
         tx = self.gdb.transaction()
         index_hits = index['test2']['test2']
@@ -1063,7 +1181,7 @@ class TransactionsTestCase(ExtensionsTestCase):
     def test_transaction_remove_node_from_index(self):
         index = self.gdb.nodes.indexes.create('index3')
         n = self.gdb.nodes.create()
-        index.add('test3','test3', n)
+        index.add('test3', 'test3', n)
         tx = self.gdb.transaction(using_globals=False)
         index.delete('test3', 'test3', n, tx=tx)
         # Assert transactional
@@ -1074,11 +1192,11 @@ class TransactionsTestCase(ExtensionsTestCase):
     def test_transaction_query_index_for_new_node(self):
         #test nodes created in transaction
 
-        index = self.gdb.nodes.indexes.create('index4%s' \
-                                              % datetime.now().strftime('%s%f'))
+        index_name = 'index4%s' % datetime.now().strftime('%s%f')
+        index = self.gdb.nodes.indexes.create(index_name)
         tx = self.gdb.transaction(using_globals=False)
         n4 = self.gdb.nodes.create(tx=tx)
-        index.add('test3','test3', n4, tx=tx)
+        index.add('test3', 'test3', n4, tx=tx)
         # Assert transactional
         transactional = True
         try:
@@ -1099,7 +1217,7 @@ class TransactionsTestCase(ExtensionsTestCase):
         n1 = self.gdb.nodes.create()
         tx = self.gdb.transaction()
         index = self.gdb.nodes.indexes.create('index5')
-        index.add('test1','test1', n1)
+        index.add('test1', 'test1', n1)
         tx.commit()
         self.assertTrue(isinstance(index, client.Index))
         self.assertTrue(index['test1']['test1'][-1] == n1)
@@ -1123,8 +1241,7 @@ class TransactionsTestCase(ExtensionsTestCase):
 
     def test_transaction_properties_class(self):
         def has_props(node):
-            return node['test1'] == 'test1' and \
-                   node['test2'] == 'test2'
+            return node['test1'] == 'test1' and node['test2'] == 'test2'
 
         def set_props(node):
             node['test1'] = 'test1'
@@ -1186,9 +1303,9 @@ class TransactionsTestCase(ExtensionsTestCase):
                     edge.relationships.create("EDGE_TAG", tag1)
                     edge.relationships.create("EDGE_TAG", tag2)
                     RUN.relationships.create("RUN_EDGE", edge)
-                    EDGE_DICT[(id1,id2)] = edge
+                    EDGE_DICT[(id1, id2)] = edge
                 else:
-                    edge = EDGE_DICT[(id1,id2)]
+                    edge = EDGE_DICT[(id1, id2)]
         print i, operations
         self.assertTrue(operations >= i)
 
@@ -1204,8 +1321,8 @@ class TransactionsTestCase(ExtensionsTestCase):
         s = self.gdb.node.create(id=1)
         d = self.gdb.node.create(id=2)
         nidx = self.gdb.nodes.indexes.create('nodelist')
-        nidx.add('nid',1, s)
-        nidx.add('nid',2, d)
+        nidx.add('nid', 1, s)
+        nidx.add('nid', 2, d)
         nodelist = [(1, 2)]
         with self.gdb.transaction():
             for s_id, d_id in nodelist:
@@ -1242,6 +1359,41 @@ class TransactionsTestCase(ExtensionsTestCase):
         rel_ca = c.relationships.outgoing()[0]
         assert(rel_ca.start == c and rel_ca.end == a)
 
+    # def test_transaction_integration(self):
+    #     """
+    #     1. Create a new node.
+    #     2. Add it to an index if no entry exists already, otherwise fail the
+    #         transaction.
+    #     3. Link the newly created node to other existing nodes by looking up
+    #         the existing nodes in other indexes.
+    #     4. Index the newly created relationship linking the new node and the
+    #         existing nodes.
+    #     """
+    #     nidx = self.gdb.nodes.indexes.create('nodes')
+    #     ridx = self.gdb.nodes.indexes.create('rels')
+    #     nidx2 = self.gdb.nodes.indexes.create('nodes2')
+    #     for i in range(3):
+    #         nidx2['key2']['val2'] = self.gdb.nodes.create()
+
+    #     def use_case():
+    #         rollback = False
+    #         with self.gdb.transaction(commit=False) as tx:
+    #             n1 = self.gdb.nodes.create()
+    #             if n1 in nidx['key']['val']:
+    #                 rollback = True
+    #             else:
+    #                 nidx['key']['val'] = n1
+    #                 for n in nidx2['key2']['val2']:
+    #                     rel = n1.relationships.create("rel", n)
+    #                     ridx['key']['val'] = rel
+    #         if not rollback:
+    #             tx.commit()
+    #             return n1
+    #         return None
+
+    #     self.assertTrue(use_case() in nidx['key']['val'])
+    #     self.assertEqual(use_case(), None)
+
 
 class PickleTestCase(TransactionsTestCase):
 
@@ -1260,6 +1412,7 @@ class PickleTestCase(TransactionsTestCase):
 
 class QueryTestCase(PickleTestCase):
 
+    @versions(not_supported=["1.6.3"])
     def test_query_raw(self):
         n1 = self.gdb.nodes.create(name="John")
         n2 = self.gdb.nodes.create(name="William")
@@ -1268,6 +1421,7 @@ class QueryTestCase(PickleTestCase):
         result = self.gdb.query(q=q)
         self.assertTrue(result is not None)
 
+    @versions(not_supported=["1.6.3"])
     def test_query_raw_returns(self):
         n1 = self.gdb.nodes.create(name="John")
         n2 = self.gdb.nodes.create(name="William")
@@ -1279,6 +1433,7 @@ class QueryTestCase(PickleTestCase):
         self.assertTrue(isinstance(m1, client.Node))
         self.assertTrue(isinstance(m2, client.Node))
 
+    @versions(not_supported=["1.6.3"])
     def test_query_raw_returns_tuple(self):
         n1 = self.gdb.nodes.create(name="John")
         n2 = self.gdb.nodes.create(name="William")
@@ -1294,6 +1449,7 @@ class QueryTestCase(PickleTestCase):
             self.assertEqual(rel, r)
             self.assertEqual(date, 1982)
 
+    @versions(not_supported=["1.6.3"])
     def test_query_params_returns_tuple(self):
         n1 = self.gdb.nodes.create(name="John")
         n2 = self.gdb.nodes.create(name="William")
@@ -1316,6 +1472,7 @@ class QueryTestCase(PickleTestCase):
 
 class FilterTestCase(QueryTestCase):
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_nodes(self):
         Q = query.Q
         for i in range(5):
@@ -1324,6 +1481,7 @@ class FilterTestCase(QueryTestCase):
         williams = self.gdb.nodes.filter(lookup)
         self.assertTrue(len(williams) >= 5)
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_nodes_complex_lookups(self):
         Q = query.Q
         for i in range(5):
@@ -1334,8 +1492,9 @@ class FilterTestCase(QueryTestCase):
              ~Q("surname", endswith="1"))
         )
         williams = self.gdb.nodes.filter(lookups)
-        self.assertTrue(len(williams) >= 5)
+        self.assertTrue(len(williams) >= 4)
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_nodes_slicing(self):
         Q = query.Q
         for i in range(5):
@@ -1344,6 +1503,7 @@ class FilterTestCase(QueryTestCase):
         williams = self.gdb.nodes.filter(lookup)[:4]
         self.assertTrue(len(williams) == 4)
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_nodes_ordering(self):
         Q = query.Q
         for i in range(5):
@@ -1353,6 +1513,7 @@ class FilterTestCase(QueryTestCase):
                                                           constants.DESC)
         self.assertTrue(williams[-1]["code"] > williams[0]["code"])
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_nodes_nullable(self):
         Q = query.Q
         for i in range(5):
@@ -1361,6 +1522,7 @@ class FilterTestCase(QueryTestCase):
         williams = self.gdb.nodes.filter(lookup)[:10]
         self.assertTrue(len(williams) > 5)
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_nodes_start(self):
         Q = query.Q
         nodes = []
@@ -1370,6 +1532,7 @@ class FilterTestCase(QueryTestCase):
         williams = self.gdb.nodes.filter(lookup, start=nodes)
         self.assertTrue(len(williams) == 5)
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_nodes_start_index(self):
         Q = query.Q
         t = unicode(datetime.now().strftime('%s%f'))
@@ -1384,6 +1547,7 @@ class FilterTestCase(QueryTestCase):
         williams = self.gdb.nodes.filter(lookup, start=index["name"])
         self.assertTrue(len(williams) == 5)
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_index_for_nodes(self):
         Q = query.Q
         n1 = self.gdb.nodes.create(name="Lemmy", band="Motörhead")
@@ -1397,6 +1561,7 @@ class FilterTestCase(QueryTestCase):
         self.assertTrue(n1 in index["bandś"].filter(lookup))
         self.assertTrue(n1 in index["bandś"].filter(lookup, value="Motörhead"))
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_index_for_relationships(self):
         Q = query.Q
         n1 = self.gdb.nodes.create(name="Jóhn Doe", place="Texaś")
@@ -1412,6 +1577,7 @@ class FilterTestCase(QueryTestCase):
         self.assertTrue(r1 in index["feelińg"].filter(lookup))
         self.assertTrue(r1 in index["feelińg"].filter(lookup, value="háte"))
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_relationships_start(self):
         Q = query.Q
         rels = []
@@ -1424,6 +1590,7 @@ class FilterTestCase(QueryTestCase):
         old_loves = self.gdb.relationships.filter(lookup, start=rels)
         self.assertTrue(len(old_loves) >= 5)
 
+    @versions(not_supported=["1.6.3"])
     def test_filter_relationships_start_index(self):
         Q = query.Q
         t = unicode(datetime.now().strftime('%s%f'))
@@ -1489,4 +1656,6 @@ class XtraCacheTestCase(unittest.TestCase):
 if __name__ == '__main__':
     test_loader = unittest.TestLoader()
     suite = test_loader.loadTestsFromTestCase(Neo4jPythonClientTestCase)
-    unittest.TextTestRunner(verbosity=2).run(suite)
+    run = unittest.TextTestRunner(verbosity=2).run(suite)
+    if run.errors or run.failures:
+        exit(1)
